@@ -47,34 +47,32 @@ class PyRunner:
     self.engine = ExecutionEngine()
     
     self._init_params = {
-      'restart' : False,
-      'config_file' : None,
-      'proc_file' : None,
-      'exec_proc_name' : None,
-      'exec_only_list' : [],
+      'config_file'       : kwargs.get('config_file'),
+      'proc_file'         : kwargs.get('proc_file'),
+      'restart'           : False,
+      'cvar_list'         : [],
+      'exec_proc_name'    : None,
+      'exec_only_list'    : [],
       'exec_disable_list' : [],
-      'exec_from_id' : None,
-      'exec_to_id' : None
+      'exec_from_id'      : None,
+      'exec_to_id'        : None
     }
     
+    self._on_create_func = None
     self._on_start_func = None
     self._on_restart_func = None
     self._on_success_func = None
     self._on_fail_func = None
-    self._on_exit_func = None
+    self._on_destroy_func = None
     
-    # Config wiring
-    self.source_config_file = self.config.source_config_file
-    
-    # Backwards compatability
-    self.load_proc_list_file = self.load_from_file
-    self.load_last_failed = self.load_state
-    
-    if kwargs.get('parse_args', False) == True:
-      self.parse_args()
+    self.parse_args()
   
-  def load_from_file(self, proc_file, restart=False):
-    if not os.path.isfile(proc_file):
+  def reset_env(self):
+    os.environ.clear()
+    os.environ.update(self._environ)
+  
+  def load_proc_file(self, proc_file, restart=False):
+    if not proc_file or not os.path.isfile(proc_file):
       return False
     
     self.register = self.serde_obj.deserialize(proc_file, restart)
@@ -123,6 +121,9 @@ class PyRunner:
     if not isinstance(obj, notification.Notification): raise Exception('Notification plugin must implement the Notification interface')
     self.notification = obj
   
+  # App lifecycle hooks
+  def on_create(self, func):
+    self._on_create_func = func
   def on_start(self, func):
     self._on_start_func = func
   def on_restart(self, func):
@@ -131,22 +132,19 @@ class PyRunner:
     self._on_success_func = func
   def on_fail(self, func):
     self._on_fail_func = func
-  def on_exit(self, func):
-    self._on_exit_func = func
+  def on_destroy(self, func):
+    self._on_destroy_func = func
   
-  def execute(self):
-    return self.run()
-  def run(self):
-    # Source config
-    self.source_config_file(self._init_params['config_file'])
-    
+  def prepare(self):
     # Initialize NodeRegister
     if self._init_params['restart']:
-      if not self.load_last_failed():
-        self._init_params['restart'] = False
-        self.load_proc_list_file(self._init_params['proc_file'])
-    else:
-      self.load_proc_list_file(self._init_params['proc_file'])
+      self.load_state()
+    elif self._init_params.get('proc_file'):
+      self.load_proc_file(self._init_params['proc_file'])
+    
+    # Inject Context var overrides
+    for k,v in self._init_params['cvar_list']:
+      self.engine.context.set(k, v)
     
     # Modify NodeRegister
     if self._init_params['exec_proc_name']:
@@ -159,15 +157,23 @@ class PyRunner:
       self.exec_from(self._init_params['exec_from_id'])
     if self._init_params['exec_to_id'] is not None:
       self.exec_to(self._init_params['exec_to_id'])
+  
+  def execute(self):
+    return self.run()
+  def run(self):
+    self.prepare()
+    
+    # App lifecycle - RESTART
+    if self._init_params['restart']:
+      if self._on_restart_func: self._on_restart_func()
+    # App lifecycle - CREATE
+    else:
+      if self._on_create_func: self._on_create_func()
     
     # App lifecycle - START
     if self._on_start_func:
       self._on_start_func()
     
-    # App lifecycle - RESTART
-    if self._init_params['restart'] and self._on_restart_func:
-      self._on_restart_func()
-    
     self.config['app_start_time'] = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
     
     # Prepare engine
@@ -201,9 +207,9 @@ class PyRunner:
         print('Skipping Email Notification: Property "email_on_fail" is set to FALSE.')
         emit_notification = False
     
-    # App lifecycle - EXIT
-    if self._on_exit_func:
-      self._on_exit_func()
+    # App lifecycle - DESTROY
+    if self._on_destroy_func:
+      self._on_destroy_func()
     
     if emit_notification:
       self.notification.emit_notification(self.config, self.register)
@@ -312,7 +318,7 @@ class PyRunner:
     return
   
   def load_state(self):
-    if not self.load_from_file(self.config.ctllog_file, True):
+    if not self.load_proc_file(self.config.ctllog_file, True):
       return False
     
     if not os.path.isfile(self.config.ctx_file):
@@ -335,6 +341,13 @@ class PyRunner:
     if os.path.isfile(self.config.ctx_file):
       os.remove(self.config.ctx_file)
   
+  def is_restartable(self):
+    if not os.path.isfile(self.config.ctllog_file):
+      return False
+    if not os.path.isfile(self.config.ctx_file):
+      return False
+    return True
+  
   # NodeRegister wiring
   def add_node(self, **kwargs)    : return self.register.add_node(**kwargs)
   def exec_only(self, id_list)    : return self.register.exec_only(id_list)
@@ -391,13 +404,11 @@ class PyRunner:
         os.environ[parts[0]] = parts[1]
       elif opt == '--cvar':
         parts = arg.split('=')
-        self.engine.context.set(parts[0], parts[1])
+        self._init_params['cvar_list'].append((parts[0], parts[1]))
       elif opt == '--nozip':
         self.config['nozip'] = True
       elif opt == '--dump-logs':
         self.config['dump_logs'] = True
-      #elif opt == '--context':
-      #  ctx_file = arg
       elif opt == '--dryrun':
         self.config['dryrun'] = True
       elif opt in ['-i', '--interactive']:
@@ -423,6 +434,16 @@ class PyRunner:
         sys.exit(0)
       else:
         raise ValueError("Error during parsing of opts")
+    
+    # We need to check for and source the app_profile/config file ASAP,
+    # but only after --env vars are processed
+    if not self._init_params.get('config_file'):
+      raise RuntimeError('Config file (app_profile) has not been provided')
+    self.config.source_config_file(self._init_params['config_file'])
+    
+    # Check if restart is possible (ctllog/ctx files exist)
+    if self._init_params['restart'] and not self.is_restartable():
+      self._init_params['restart'] = False
   
   def show_help(self):
     print("Required:")
